@@ -8,6 +8,8 @@
 #define LOCAL_DEBUG
 #include "debug.h"
 
+#include "acsyscap.h"
+
 #include "meta.h"
 #include "header.h"
 #include "caddrinfo.h"
@@ -18,6 +20,9 @@
 #include "cleaner.h"
 #include "tcpconnect.h"
 
+#include "fileitem.h"
+#include "dlcon.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
@@ -34,7 +39,6 @@
 #include <iostream>
 #include <list>
 
-#include "acsyscap.h"
 
 #define FUSE_USE_VERSION 25
 #include <fuse.h>
@@ -63,65 +67,48 @@ using namespace std;
 
 const time_t cleaner::never=ULONG_MAX;
 
-// keep reused code quiet
-namespace aclog
-{
-void err(const char*, const char*)
-{}
-}
-
 // some globals, set only once
 static struct stat statTempl;
 static struct statfs stfsTemp;
 static tHttpUrl baseUrl, proxyUrl;
 static mstring altPath;
+bool g_bGoodServer=true;
 
 struct tDlDesc
 {
-	off_t seen_length;
-	int seen_status;
-	acbuf databuf;
+	cmstring m_path;
+	UINT m_ftype;
 
-	//virtual bool CheckOrExplore(const char *path) =0;
-	virtual bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false) =0;
-
-	tDlDesc() : seen_length(0), seen_status(0) {};
+	virtual int Read(char *retbuf, const char *path, off_t pos, size_t len) =0;
+	virtual int Stat(struct stat &stbuf) =0;
+	tDlDesc(cmstring &p, UINT ftype) : m_path(p), m_ftype(ftype) {};
 	virtual ~tDlDesc() {};
 };
 
 struct tDlDescLocal : public tDlDesc
 {
-	static tDlDesc* Create(const char *path)
-	{
-		tDlDescLocal *p(NULL);
-		MYTRY
-		{
-			p = new tDlDescLocal;
-			if (!p) // running exception-free?
-				return NULL;
-		}
-		MYCATCH(...)
-		{
-			return NULL;
-		}
-
-		FILE *pf = fopen(path, "rb");
-		if (!pf)
-		{
-			delete p;
-			return NULL;
-		}
-
-		p->seen_status = 200;
-		fseeko(pf, 0, SEEK_END);
-		p->seen_length = ftello(pf);
-
-		p->pFile = pf;
-
-		return p;
-	}
 	FILE *pFile;
-	tDlDescLocal() : pFile(NULL) {};
+	tDlDescLocal(cmstring &path, UINT ftype) : tDlDesc(path, ftype), pFile(NULL)
+	{
+	};
+
+	int Stat(struct stat &stbuf)
+	{
+		if(altPath.empty()) // hm?
+			return -ENOENT;
+
+		if (::stat((altPath + m_path).c_str(), &stbuf))
+			return -errno;
+
+		// verify the file state
+		header h;
+		int r = h.LoadFromFile(altPath + m_path + ".head");
+		if (r <= 0 || stbuf.st_size != atoofft(h.h[header::CONTENT_LENGTH], -23))
+			return -EIO;
+
+		return 0;
+	}
+
 	virtual ~tDlDescLocal()
 	{
 		if(pFile)
@@ -129,91 +116,285 @@ struct tDlDescLocal : public tDlDesc
 		pFile=NULL;
 	};
 
-	bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false)
+	int Read(char *retbuf, const char *path, off_t pos, size_t len)
 	{
-		databuf.clear();
-		if(databuf.freecapa() < len)
-			databuf.init(len);
+		if (!pFile)
+		{
+			struct stat stbuf;
+			if(Stat(stbuf))
+				return -EIO; // file incomplete or missing
 
+			FILE *pf = fopen((altPath + m_path).c_str(), "rb");
+			if (!pf)
+				return -EIO;
+			pFile = pf;
+		}
+
+		int copied=0;
 		if(pFile && 0==fseeko(pFile, pos, SEEK_SET))
 		{
-			while(!feof(pFile) && !ferror(pFile) && databuf.size() <len)
+			while(!feof(pFile) && !ferror(pFile) && len>0)
 			{
-				size_t r = ::fread(databuf.wptr(), 1, len-databuf.size(), pFile);
-				databuf.got(r);
+				size_t r = ::fread(retbuf+copied, 1, len, pFile);
+				copied+=r;
+				len-=r;
 			}
 		}
-
-		if(databuf.size() == len || seen_length == (pos+databuf.size()))
-		{
-			seen_status=206;
-			return true;
-		}
-
-		return false;
+		return ferror(pFile) ? -EIO : copied;
 	}
 };
 
+static lockable mxCache;
+struct tFileId
+{ off_t m_size; mstring m_ctime;
+tFileId() : m_size(0) {};
+tFileId(off_t a, mstring b) : m_size(a), m_ctime(b) {};
+bool operator!=(tFileId other) const { return m_size != other.m_size || m_ctime != other.m_ctime;}
+};
+static map<string, tFileId> remote_info_cache;
+
 struct tDlDescRemote : public tDlDesc
 {
-	header h;
+protected:
 
-	tDlDescRemote()
+	tFileId fid;
+	bool bIsFirst; // hint to catch the validation data when download starts
+
+public:
+	tDlDescRemote(cmstring &p, UINT n) : tDlDesc(p,n), bIsFirst(true)
 	{
 		// expire the caches every time, should not cost much anyway
 		tcpconnect::ExpireCache();
 		CAddrInfo::ExpireCache();
 	};
-	virtual ~tDlDescRemote()
+
+	int Read(char *retbuf, const char *path, off_t pos, size_t len)
 	{
-	}
-	
-	bool CheckOrExplore(const char *path)
-	{
-		static lockable mxCache;
-		static map<string, int> cache;
+		dlcon dler(true, 0);
+		tHttpUrl uri = proxyUrl;
+		uri.sPath += baseUrl.sHost
+		// + ":" + ( baseUrl.sPort.empty() ? baseUrl.sPort : "80")
+				+ baseUrl.sPath + m_path;
+		class tFitem: public fileitem_base
+		{
+		public:
+			char *pRet;
+			size_t nRest, nGot;
+			off_t skipBytes;
+			int nErr;
+
+			ssize_t SendData(int, int, off_t&, size_t)
+			{
+				return 0;
+			} // nothing to send
+			bool StoreFileData(const char *p, unsigned int count)
+			{
+				if (count == 0)
+				{
+					m_status=FIST_COMPLETE;
+					return true;
+				}
+
+				if(skipBytes>0)
+				{
+					if(skipBytes>count)
+					{
+						skipBytes-=count;
+						return true;
+					}
+					count-=skipBytes;
+					p+=skipBytes;
+					skipBytes=0;
+				}
+
+				if(!nRest)
+				{
+					m_status=FIST_COMPLETE;
+					return false;
+				}
+				if(count>nRest)
+					count=nRest;
+				memcpy(pRet+nGot, p, count);
+				nGot+=count;
+				nRest-=count;
+				return true;
+			}
+#define SETERROR { nErr=__LINE__; return false;}
+			bool &m_isFirst;
+			bool DownloadStartedStoreHeader(const header &head, const char*)
+			{
+				_cerr(head.frontLine<<endl);
+				m_head = head; // XXX: bloat, only status line and contlen required
+				int st =head.getStatus();
+
+				if(st == 416)
+					return true; // EOF
+
+				if(st != 200 && st != 206)
+				{
+					SETERROR;
+				}
+
+				// validation
+				if (head.h[header::LAST_MODIFIED])
+				{
+					if (m_isFirst)
+						fid.m_ctime = head.h[header::LAST_MODIFIED];
+					else if (fid.m_ctime != head.h[header::LAST_MODIFIED])
+						SETERROR;
+				}
+
+				off_t myfrom(0), myto(0), mylen(0);
+				const char *p=head.h[header::CONTENT_RANGE];
+				if(p)
+				{
+					int n=sscanf(p, "bytes " OFF_T_FMT "-" OFF_T_FMT "/" OFF_T_FMT, &myfrom, &myto, &mylen);
+					if(n<=0)
+						n=sscanf(p, "bytes=" OFF_T_FMT "-" OFF_T_FMT "/" OFF_T_FMT, &myfrom, &myto, &mylen);
+					if(n!=3  // check for nonsense
+							|| (m_nSizeSeen>0 && myfrom != m_nSizeSeen-1)
+							|| (m_nRangeLimit>=0 && myto > m_nRangeLimit) // too much data?
+							|| myfrom<0 || mylen<0
+					)
+					{
+						SETERROR;
+					}
+
+				}
+				else if(st == 200 && head.h[header::CONTENT_LENGTH])
+					mylen = atoofft(head.h[header::CONTENT_LENGTH]);
+
+				// validation
+				if(m_isFirst)
+					fid.m_size = mylen;
+				else
+					if(fid.m_size != mylen)
+						SETERROR;
+
+				skipBytes -= myfrom;
+				if(skipBytes<0)
+					SETERROR;
+				return true;
+			}
+			tFileId &fid;
+			tFitem(char *p, size_t size, off_t start, tFileId &fi, bool &isfirst)
+			: pRet(p), nRest(size),
+					nGot(0), skipBytes(start), nErr(0), m_isFirst(isfirst), fid(fi)
+			{
+				m_bCheckFreshness = false;
+				m_nSizeSeen = start;
+				m_nRangeLimit = g_bGoodServer ? start+size-1 : -1;
+			}
+		};
+
 		{
 			lockguard g(mxCache);
-			map<string,int>::iterator it=cache.find(path);
-			if(it!=cache.end())
+			map<string, tFileId>::const_iterator it = remote_info_cache.find(path);
+			if (it != remote_info_cache.end())
+				fid = it->second;
+		}
+		tFileId fidOrig=fid;
+
+		tFitem *pFi = new tFitem(retbuf, len, pos, fid, bIsFirst);
+		tFileItemPtr spFi(static_cast<fileitem_base*>(pFi));
+		dler.AddJob(spFi, uri);
+		dler.WorkLoop();
+		int nHttpCode(100);
+		pFi->WaitForFinish(&nHttpCode);
+		bIsFirst=false;
+
+
+		if (m_ftype == FILE_PKG && fidOrig != fid)
+		{
+			lockguard g(mxCache);
+			remote_info_cache[m_path] = fid;
+		}
+
+		if(nHttpCode==416)
+			return 0; // EOF
+		if(pFi->nErr || !pFi->nGot)
+			return -EIO;
+		return pFi->nGot;
+	}
+
+	int Stat(struct stat &stbuf)
+	{
+		stbuf = statTempl;
+		{
+			lockguard g(mxCache);
+			map<string, tFileId>::const_iterator it = remote_info_cache.find(m_path);
+			if (it != remote_info_cache.end())
 			{
-				seen_length=it->second;
-				seen_status=200;
+				stbuf.st_size = it->second.m_size;
+				stbuf.st_mode &= ~S_IFDIR;
+				stbuf.st_mode |= S_IFREG;
+				struct tm tmx;
+				if(header::ParseDate(it->second.m_ctime.c_str(), &tmx))
+					stbuf.st_ctime = mktime(&tmx);
 				_cerr("Using precached\n");
-				return true;
+				return 0;
 			}
 		}
 		// ok, not cached, do the hard way
-		bool res=Act(path, 0, 1, true);
-		_cerr("Act done, result: "<<res<<endl);
-		if(!res)
-			return res;
-		UINT type=rechecks::GetFiletype(path);
-		_cerr ( "type: " << type );
-		if (type!=FILE_INDEX) // not caching volatile stuff
+
+		dlcon dler(true, 0);
+
+		tHttpUrl uri = proxyUrl;
+		uri.sPath += baseUrl.sHost
+		// + ":" + ( baseUrl.sPort.empty() ? baseUrl.sPort : "80")
+				+ baseUrl.sPath + m_path;
+		class tFitemProbe: public fileitem_base
 		{
-			lockguard g(mxCache);
-			cache[path]=seen_length;
+		public:
+			ssize_t SendData(int, int, off_t&, size_t)
+			{
+				return 0;
+			} // nothing to send
+			bool StoreFileData(const char*, unsigned int)
+			{
+				return false;
+			}
+			tFitemProbe()
+			{
+				m_bHeadOnly = true;
+			}
+			bool DownloadStartedStoreHeader(const header &head, const char*)
+			{
+				m_head = head; // XXX: bloat, only status line and contlen required
+				m_status = FIST_COMPLETE;
+				return true;
+			}
+		};
+		tFileItemPtr probe(static_cast<fileitem_base*>(new tFitemProbe()));
+		dler.AddJob(probe, uri);
+		dler.WorkLoop();
+		int nHttpCode(100);
+		FiStatus res = probe->WaitForFinish(&nHttpCode);
+		stbuf.st_size = atoofft(probe->GetHeaderUnlocked().h[header::CONTENT_LENGTH], 0);
+		stbuf.st_mode &= ~S_IFDIR;
+		stbuf.st_mode |= S_IFREG;
+
+		if (res < FIST_COMPLETE)
+			return -EIO;
+		else if (nHttpCode == 200)
+		{
+			if (m_ftype == FILE_PKG) // not caching volatile stuff
+			{
+				lockguard g(mxCache);
+				remote_info_cache[m_path] =
+						tFileId(stbuf.st_size, probe->GetHeaderUnlocked().h[header::LAST_MODIFIED]);
+			}
+			struct tm tmx;
+			if(header::ParseDate(probe->GetHeaderUnlocked().h[header::LAST_MODIFIED], &tmx))
+				stbuf.st_ctime = mktime(&tmx);
+			return 0;
 		}
-		return true;
-	}
-	
-
-	inline int WaitForResponse(int fd)
-	{
-	// prevent the thing from looping too fast before server
-		fd_set rfds;
-		FD_ZERO(&rfds);
-		FD_SET(fd, &rfds);
-		struct timeval tv;
-		tv.tv_sec = 5;
-		tv.tv_usec = 0;
-		return select(fd + 1, &rfds, NULL, NULL, &tv);
+		return -ENOENT;
 	}
 
-
-	bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false)
+	bool Act(const char *path, off_t pos, off_t len)
 	{
+		/*
 		_cerr( path << ", from: " << pos << " , " << len << "bytes\n");
 		databuf.clear();
 		h.clear();
@@ -368,134 +549,38 @@ struct tDlDescRemote : public tDlDesc
 		}
 		com_done_con_idle:
 		tcpconnect::RecycleIdleConnection(con);
+
 		return true;
-	};
-	/*
-	inline void _ShutDown()
-	{
-		_cerr("Disconnecting...\n");
-		if (fd>=0)
-		{
-			shutdown(fd, O_RDWR);
-			close(fd);
-		}
-		fd=-1;
-	};
-	bool _Connect(bool bForceFresh)
-	{
-		if(fd>=0)
-			return true;
-
-		if(!bForceFresh)
-		{
-			lockguard g(mx);
-			// also do some pool maintenance... too large or too old? nuke it
-			if(fdpool.size()>POOLMAXSIZE || fdpool.back().first < (time(0)-POOLMAXAGE))
-			{
-				while(!fdpool.empty())
-				{
-					int zombie=fdpool.front().second;
-					::shutdown(zombie, O_RDWR);
-					::close(zombie);
-					fdpool.pop_front();
-				}
-			}
-			if(!fdpool.empty())
-			{
-				fd=fdpool.back().second;
-				fdpool.pop_back();
-				return true;
-			}
-		}
-
-		signal(SIGPIPE, SIG_IGN);
-		string sErr;
-    CAddrInfo::SPtr dns=CAddrInfo::CachedResolve(proxyUrl.sHost, proxyUrl.sPort, sErr);
-		if(!dns)
-		{
-			sErr="500 Cannot resolve remote address.";
-			return false;
-		}
-		struct addrinfo *pInfo(dns->m_bestInfo);
-		fd = socket(pInfo->ai_family, pInfo->ai_socktype, pInfo->ai_protocol);
-		if(fd<0)
-			return false;
-
-#ifndef NO_TCP_TUNNING
-		int dummy(1);
-		setsockopt(fd,SOL_SOCKET,SO_REUSEADDR,&dummy,sizeof(dummy));
-		setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &dummy, sizeof(dummy));
-#endif
-		int r=connect(fd, pInfo->ai_addr, pInfo->ai_addrlen);
-		if (r<0)
-		{
-			sErr="500 Connection failure";
-			return false;
-		}
-		return true;
-	}
-*/
-};
-
-bool CheckLocalFile(const char *path, struct stat *stbuf)
-{
-	//ldbg("Is a file?!");
-
-	if (stat((altPath + path).c_str(), stbuf))
+		*/
 		return false;
 
-	header h;
-	int r = h.LoadFromFile(altPath + path + ".head");
-	if (r > 0)
-	{
-		const char *szLen = h.h[header::CONTENT_LENGTH];
-		if (szLen && *szLen && atoofft(szLen) == stbuf->st_size)
-		{
-			// ok, it's static and length is ok?
-			return true;
-		}
-	}
+	};
+};
 
-	return false;
-}
 
 
 /// If found as downloadable, present as a file, or become a directory otherwise.
 static int acngfs_getattr(const char *path, struct stat *stbuf)
 {
-	// assume that everything downloadable is a file and a directory is 404
+	if(!path)
+		return -1;
 
 	UINT type = rechecks::GetFiletype(path);
 	_cerr( "type: " << type);
 	if (type == FILE_PKG || type == FILE_INDEX)
 	{
-		if (!altPath.empty() && type == FILE_PKG && CheckLocalFile(path, stbuf))
+		if(0 == tDlDescLocal(path, type).Stat(*stbuf))
 			return 0;
-
-		memcpy(stbuf, &statTempl, sizeof(statTempl));
-		stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
-		stbuf->st_mode |= S_IFREG;
-		stbuf->st_size = 0;
-
-		tDlDescRemote e;
-		if (e.CheckOrExplore(path) && e.seen_status == 200)
-		{
-			stbuf->st_size = e.seen_length;
+		if(0 == tDlDescRemote(path, type).Stat(*stbuf))
 			return 0;
-		}
-
-		return -EIO;
 	}
-	else
-	{
-		//ldbg("Is a directory!");
-		memcpy(stbuf, &statTempl, sizeof(statTempl));
-		stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
 
-		stbuf->st_mode |= S_IFDIR;
-		stbuf->st_size = 4;
-		return 0;
-	}
+	//ldbg("Be a directory!");
+	memcpy(stbuf, &statTempl, sizeof(statTempl));
+	stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
+	stbuf->st_mode |= S_IFDIR;
+	stbuf->st_size = 4;
+	return 0;
 }
 
 static int acngfs_fgetattr(const char *path, struct stat *stbuf,
@@ -606,52 +691,40 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 
 	tDlDesc *p(NULL);
 	struct stat stbuf;
-
-	// ok... if that is a remote object, can we still use local access instead?
-	if(!altPath.empty() && FILE_PKG == rechecks::GetFiletype(path)
-			 && CheckLocalFile(path, &stbuf))
-	{
-		p = tDlDescLocal::Create((altPath+path).c_str());
-		if(p)
-			goto desc_opened;
-	}
-
+	UINT ftype = rechecks::GetFiletype(path);
 
 	MYTRY
 	{
-		p=new tDlDescRemote;
+		// ok... if that is a remote object, can we still use local access instead?
+		if(!altPath.empty() && FILE_PKG == ftype)
+		{
+				p = new tDlDescLocal(path, ftype);
+				if(p)
+				{
+					if(0==p->Stat(stbuf))
+						goto desc_opened;
+					delete p;
+					p=NULL;
+				}
+		}
+
+
+		p=new tDlDescRemote(path, ftype);
 		if(!p) // running exception-free? 
-			return -EIO;	
+			return -EIO;
+		if(0!=p->Stat(stbuf))
+		{
+			delete p;
+			return -EIO;
+		}
 	}
-	MYCATCH(...)
+	MYCATCH(std::bad_alloc)
 	{
 		return -EIO;
 	}
 	
-	if( ! static_cast<tDlDescRemote*>(p)->CheckOrExplore(path) || p->seen_status !=200)
-	{
-		delete p;
-
-		// last alternative :-(
-		if(!altPath.empty())
-		{
-			p=tDlDescLocal::Create( (altPath+path).c_str());
-			if(p)
-				goto desc_opened;
-		}
-
-		return -EISDIR;
-	}
-
-
 	desc_opened:
 
-	/*
-	long size = p->h.h[header::CONTENT_LENGTH] ? atol(p->h.h[header::CONTENT_LENGTH]) : 0;  
-	if(size) // precache
-		p->Act(path, size-1, 1);
-	*/
-	
 	fi->fh = (uintptr_t) p;
 	return 0;
 }
@@ -661,27 +734,11 @@ static int acngfs_read(const char *path, char *buf, size_t size, off_t offset,
       struct fuse_file_info *fi)
 {
 	tDlDesc *p=(tDlDesc*) fi->fh;
-	_cerr( offset << ":"<<size<<":"<<p->seen_length);
-	if( off_t(offset+size) > p->seen_length)
-		size=p->seen_length-offset;
+	//_cerr( offset << ":"<<size<<":"<<p->seen_length);
+	//if( off_t(offset+size) > p->seen_length)
+	//	size=p->seen_length-offset;
 		
-	if(!p->Act(path, offset, size))
-		return -EIO;
-	
-	int s=p->seen_status;
-	_cerr( "read, status: " << s);
-	//if( ( 200==s && 0==offset) || (206==s && 0!=offset))
-
-	// 200 is possible if file is smaller than the first range
-	if(206==s || (200==s && 0==offset && off_t(size) < p->seen_length))
-	{
-		int len=MIN(size, p->databuf.size());
-		_cerr( "ret, len: " << len );
-		memcpy(buf, p->databuf.rptr(), len);
-		return len;
-	}
-	else
-		return -EIO; 
+	return p->Read(buf, path, offset, size);
 }
 
 static int acngfs_write(const char *path, const char *buf, size_t size,
@@ -751,7 +808,6 @@ void _ExitUsage() {
 
 int main(int argc, char *argv[])
 {
-
    memset(&acngfs_oper, 0, sizeof(acngfs_oper));
 
    acngfs_oper.getattr	= acngfs_getattr;
@@ -790,6 +846,14 @@ int main(int argc, char *argv[])
    if(argc<4)
       barf("Not enough arguments, try --help.\n");
    
+   acfg::agentname = "ACNGFS";
+   acfg::agentheader="User-Agent: ACNGFS\r\n";
+   acfg::requestapx = "User-Agent: ACNGFS\r\nX-Original-Source: 42\r\n";
+#ifdef SPAM
+   acfg::debug=0xff;
+   acfg::verboselog=1;
+#endif
+
    if(argv[1] && baseUrl.SetHttpUrl(argv[1]))
    {
 #ifdef VERBOSE
@@ -814,7 +878,23 @@ int main(int argc, char *argv[])
 	   cerr << "Invalid proxy URL, " << argv[2] <<endl;
 	   exit(EXIT_FAILURE);
    }
+
+   // all parameters processed, forwarded to fuse call below
+
+   ::rechecks::CompileExpressions();
    
+#if 0//def SPAM
+   {
+	   fuse_file_info fi = {0};
+	   const char *dingsda="/dists/unstable/InRelease";
+	   acngfs_open(dingsda, &fi);
+	   char buf[165536];
+	   off_t pos=0;
+	   for(;0 < acngfs_read(dingsda, buf, sizeof(buf), pos, &fi); pos+=sizeof(buf)) ;
+	   return 0;
+   }
+#endif
+
    unsigned int nMyArgCount = 2; // base url, proxy host
    // alternative path supplied in the next argument?
    if(argc > 4 && argv[4] && argv[4][0] != '-' ) // 4th argument is not an option?
@@ -822,6 +902,8 @@ int main(int argc, char *argv[])
 	   nMyArgCount=3;
 	   altPath = argv[3];
    }
+
+
    
    // test mount point
    const char *mpoint = argv[nMyArgCount+1];
@@ -830,8 +912,6 @@ int main(int argc, char *argv[])
    if(!S_ISDIR(statTempl.st_mode))
       barf(endl<< mpoint << " is not a directory.");
 
-   ::rechecks::CompileExpressions();
-
    // skip our arguments, keep those for fuse including mount point and argv[0] at the right place
    argv[nMyArgCount]=argv[0]; // application path
    argv=&argv[nMyArgCount];
@@ -844,8 +924,18 @@ int main(int argc, char *argv[])
 namespace aclog
 {
    void flush() {};
-   void misc(const string &, const char )
+   void misc(const string &s, const char )
    {
-      // G.N.D.N.
+#ifdef SPAM
+      cerr << s << endl;
+#endif
    }
+
+void err(const char *s, const char* z)
+{
+#ifdef SPAM
+      cerr << s << endl << z << endl;
+#endif
+
+};
 }
