@@ -4,6 +4,7 @@
 
 #include "config.h"
 #include "meta.h"
+#include <bitset>
 
 class hostdata;
 
@@ -50,16 +51,6 @@ void PostProcConfig();
 				
 const mstring * GetRepNameAndPathResidual(const tHttpUrl & in, mstring & sRetPathResidual);
 
-typedef struct { 
-	const char *name; mstring *ptr;
-	const char *warn; }
-MapNameToString;
-
-typedef struct { 
-	const char *name; int *ptr;
-	const char *warn; int base;}
-MapNameToInt;
-
 struct tRepoData
 {
 	MYSTD::vector<tHttpUrl> m_backends;
@@ -83,6 +74,8 @@ time_t ExecutePostponed();
 
 extern tStrMap localdirs;
 extern NoCaseStringMap mimemap;
+#define TCP_PORT_MAX 65536
+extern MYSTD::bitset<TCP_PORT_MAX> *pUserPorts;
 
 extern mstring cacheDirSlash; // guaranteed to have a trailing path separator
 
