@@ -97,6 +97,13 @@ public:
 	tCacheMan(int);
 	virtual ~tCacheMan();
 
+	// having this here makes no sense except of working around an ICE bug of gcc 4.3/4.4.x
+	class tGetItemDelegate
+	{
+		public:
+			virtual operator tFileItemPtr()=0;
+	};
+
 protected:
 	enum enumIndexType
 	{
@@ -160,7 +167,6 @@ protected:
 
 	bool FindURL_InitDload(tFileItemPtr pFi, cmstring &cacheRefFileRel, mstring &sErr, tHttpUrl *pOrigURL);
 
-	class tGetItemDelegate;
 #define VERB_QUIET 0
 #define VERB_SHOW 1
 #define VERB_SHOW_NOERRORS 2
@@ -219,6 +225,8 @@ private:
 
 	bool IsDeprecatedArchFile(cmstring &sFilePathRel);
 
+	const tIfileAttribs attr_dummy_pure;
+	tIfileAttribs attr_dummy;
 };
 
 
