@@ -77,7 +77,7 @@ struct tFingerprint {
 	{
 		return GetCsAsString()+"_"+offttos(size);
 	}
-	bool operator==(const tFingerprint & other) const
+	inline bool operator==(const tFingerprint & other) const
 	{
 		if(other.csType!=csType || size!=other.size)
 			return false;
@@ -140,20 +140,20 @@ struct tImpFileInfo
 };
 struct ltCacheKeyComp
 {
-  bool operator()(const tImpFileInfo &a, const tImpFileInfo &b) const
+  inline bool operator()(const tImpFileInfo &a, const tImpFileInfo &b) const
   {
+	  return (a.mtime!=b.mtime) ? (a.mtime<b.mtime) :
 #if defined(WINDOWS) || defined(WIN32)
-#warning FIXME, make this use nocasestringmap
+			  (strcasecmp(a.sPath.c_str(), b.sPath.c_str()) < 0);
+#else
+			  (a.sPath<b.sPath);
 #endif
-	  if(a.mtime!=b.mtime)
-		  return a.mtime<b.mtime;
-      return (a.sPath<b.sPath);
   }
 };
 
-typedef MYSTD::map<tFingerprint, tImpFileInfo> tImportMap;
+typedef MYMAP<tFingerprint, tImpFileInfo> tImportMap;
 typedef MYSTD::deque<MYSTD::pair<tFingerprint, tImpFileInfo> > tImportList;
-typedef MYSTD::map<tImpFileInfo, tFingerprint, ltCacheKeyComp> tFprCacheMap;
+typedef MYMAP<tImpFileInfo, tFingerprint, ltCacheKeyComp> tFprCacheMap;
 
 
 #endif /*CSMAPPING_H_*/
