@@ -6,11 +6,22 @@
 class expiration : public tCacheMan
 {
 public:
-	expiration(int);
+
+	enum workType
+	{
+		expire,
+		list,
+		purge,
+		listDamaged,
+		purgeDamaged
+	};
+	expiration(int fd, workType type);
 	virtual ~expiration();
 
 protected:
 
+	workType m_mode;
+
 	tS2DAT m_trashCandSet;
 	set<tFileNdir> m_trashCandHeadSet; // just collect the list of seen head files
 
@@ -34,6 +45,8 @@ protected:
 
 	void DropExceptionalVersions();
 
+	std::ofstream m_damageList;
+	bool m_bIncompleteIsDamaged;
 };
 
 #endif /*EXPIRATION_H_*/
