@@ -15,6 +15,10 @@
 #include <sys/stat.h>
 #include <cstdio>
 
+#ifdef HAVE_LINUX_SENDFILE
+#include <sys/sendfile.h>
+#endif
+
 #ifndef O_BINARY
 #define O_BINARY 0 // ignore on Unix
 #endif
