@@ -354,5 +354,40 @@ void DelTree(cmstring &what);
 
 bool IsAbsolute(cmstring &dirToFix);
 
+
+
+inline char unEscape(const char p)
+{
+	switch (p)
+	{
+	case '0':
+		return '\0';
+	case 'a':
+		return '\a';
+	case 'b':
+		return '\b';
+	case 't':
+		return '\t';
+	case 'n':
+		return '\n';
+	case 'r':
+		return '\r';
+	case 'v':
+		return '\v';
+	case 'f':
+		return '\f';
+	default:
+		return p;
+	}
+}
+
+inline mstring unEscape(cmstring &s)
+{
+	mstring ret;
+	for(cmstring::const_iterator it=s.begin();it!=s.end();++it)
+		ret+=char(*it=='\\' ? unEscape(*it) : *it);
+	return ret;
+}
+
 #endif // _META_H
 
