@@ -8,7 +8,8 @@
 
 using namespace MYSTD;
 
-namespace acfg {
+namespace acfg
+{
 
 #if defined(__CYGWIN32__) || defined(__CYGWIN__)
 #warning Cygwin seems to have an ancient regex engine which does not distuinguish between greedy and non-greedy repetition, and defaults to non-greedy
