@@ -52,12 +52,6 @@ tCacheMan::~tCacheMan()
 	m_pDlcon=NULL;
 }
 
-class tCacheMan::tGetItemDelegate
-{
-public:
-	virtual operator tFileItemPtr()=0;
-};
-
 bool tCacheMan::ProcessOthers(const string & sPath, const struct stat &)
 {
 	// NOOP
@@ -73,6 +67,9 @@ bool tCacheMan::ProcessDirAfter(const string & sPath, const struct stat &)
 
 void tCacheMan::AddIFileCandidate(const string &sPathRel)
 {
+	if(sPathRel.empty())
+		return;
+
 	enumIndexType t;
 	if ( (FILE_INDEX == rechecks::GetFiletype(sPathRel)
 	// SUSE stuff, not volatile but also contains file index data
@@ -87,54 +84,18 @@ void tCacheMan::AddIFileCandidate(const string &sPathRel)
 
 const tCacheMan::tIfileAttribs & tCacheMan::GetFlags(cmstring &sPathRel) const
 {
-	static const tIfileAttribs def; // good enough for single thread
 	tS2IDX::const_iterator it=m_indexFilesRel.find(sPathRel);
 	if(m_indexFilesRel.end()==it)
-		return def;
+		return attr_dummy_pure;
 	return it->second;
 }
 
 tCacheMan::tIfileAttribs &tCacheMan::SetFlags(cmstring &sPathRel)
 {
-	return m_indexFilesRel[sPathRel];
+	ASSERT(!sPathRel.empty());
+	return sPathRel.empty() ? attr_dummy : m_indexFilesRel[sPathRel];
 }
 
-/*
-bool tCacheMan::DownloadRec(tFileItemPtr pFi, const char *pURL, int nTTL)
-{
-	if(--nTTL<0)
-	{
-		SendChunk("Redirection count reached, aborting!<br>");
-		return false;
-	}
-	if(!m_pDlcon || !pFi)
-		return false;
-	// get fileitem into a pristine state
-	pFi->ResetCacheState();
-	pFi->Setup(true);
-	tHttpUrl url;
-	if(!pURL || !*pURL || !url.SetHttpUrl(pURL))
-		return false;
-	m_pDlcon->AddJob(pFi, url);
-	m_pDlcon->WorkLoop();
-
-	int httpCode(0);
-	if(FIST_COMPLETE != pFi->WaitForFinish(&httpCode))
-		return false;
-
-	switch (httpCode)
-	{
-	case 200:
-		return true;
-	case 301:
-	case 302:
-		return DownloadRec(pFi, pFi->GetHeader().h[header::LOCATION]);
-	default:
-		return false;
-	}
-}
-*/
-
 bool tCacheMan::FindURL_InitDload(tFileItemPtr fi, cmstring &sFilePathRel,
 		mstring &sErr, tHttpUrl *retURL)
 {
@@ -678,39 +639,40 @@ bool tCacheMan::PatchFile(const string &srcRel,
 	return true;
 
 }
+class fakeCon : public ::tCacheMan::tGetItemDelegate
+{
+   public:
+      cmstring & sTempDataRel, &sReferencePathRel;
+      off_t nGotSize;
+      fakeCon(cmstring & a, cmstring &b) : sTempDataRel(a), sReferencePathRel(b) {};
+      operator tFileItemPtr()
+      {
+         class tHeadOnlyStorage : public fileitem_with_storage
+         {
+            public:
+               fakeCon &m_parent;
+               tHeadOnlyStorage(fakeCon &p)
+                  : fileitem_with_storage(p.sTempDataRel) // storage ref to physical data file
+                    , m_parent(p)
+            {
+               m_bAllowStoreData=false;
+               m_bHeadOnly=true;
+               m_head.LoadFromFile(SABSPATH(p.sReferencePathRel+".head"));
+            }
+               void Unreg()
+               {
+                  m_head.StoreToFile(m_sPathAbs+".head");
+                  m_parent.nGotSize = atoofft(m_head.h[header::CONTENT_LENGTH], -17);
+               };
+         };
+         return tFileItemPtr(static_cast<fileitem_base*>(new tHeadOnlyStorage(*this)));
+      }
+};
 
 bool tCacheMan::GetAndCheckHead(cmstring & sTempDataRel, cmstring &sReferencePathRel,
 		off_t nWantedSize)
 {
-	class fakeCon : public tGetItemDelegate
-	{
-	public:
-		cmstring & sTempDataRel, &sReferencePathRel;
-		off_t nGotSize;
-		fakeCon(cmstring & a, cmstring &b) : sTempDataRel(a), sReferencePathRel(b) {};
-		operator tFileItemPtr()
-		{
-			class tHeadOnlyStorage : public fileitem
-			{
-			public:
-				fakeCon &m_parent;
-				tHeadOnlyStorage(fakeCon &p)
-				: fileitem(p.sTempDataRel) // storage ref to physical data file
-				, m_parent(p)
-				{
-					m_bAllowStoreData=false;
-					m_bHeadOnly=true;
-					m_head.LoadFromFile(SABSPATH(p.sReferencePathRel+".head"));
-				}
-				void Unreg()
-				{
-					m_head.StoreToFile(m_sPathAbs+".head");
-					m_parent.nGotSize = atoofft(m_head.h[header::CONTENT_LENGTH], -17);
-				};
-			};
-			return tFileItemPtr(static_cast<fileitem*>(new tHeadOnlyStorage(*this)));
-		}
-	} contor(sTempDataRel, sReferencePathRel);
+	fakeCon contor(sTempDataRel, sReferencePathRel);
 	return Download(sReferencePathRel, true, VERB_QUIET, &contor) && contor.nGotSize == nWantedSize;
 }
 
@@ -793,7 +755,7 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 
 			if (Setup(true) >= FIST_LOCALERROR)
 				return false;
-			if (!fileitem::DownloadStartedStoreHeader(head, NULL))
+			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL))
 				return false;
 			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(NULL, 0))
 				return false;
@@ -803,7 +765,7 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 		}
 	};
 	tInjectItem *p=new tInjectItem(to, bTryLink);
-	tFileItemPtr fi(static_cast<fileitem*>(p));
+	SHARED_PTR<fileitem> fi(static_cast<fileitem*>(p));
 	if (!fi || !fileitem::RegisterFileItem(to, fi) )
 	{
 		MTLOGASSERT(false, "Couldn't register copy item");
