@@ -89,9 +89,9 @@ struct tDlJob
 			m_pStorage->IncDlRefCount();
 	}
 
-	tDlJob(dlcon *p, tFileItemPtr pFi, const string & sHost, const string & sPath) :
+	tDlJob(dlcon *p, tFileItemPtr pFi, const tHttpUrl& uri) :
 			m_pStorage(pFi), m_pBEdata(NULL),
-			m_fileUri(sHost, sPath)
+			m_fileUri(uri)
 	{
 		Init();
 	}
@@ -151,7 +151,7 @@ struct tDlJob
 		// either by backend or by host in file uri, never both
 		ASSERT( (m_pCurBackend && m_fileUri.sHost.empty()) || (!m_pCurBackend && !m_fileUri.sHost.empty()));
 
-		if (m_pStorage->m_nSizeSeen > 0)
+		if (m_pStorage->m_nSizeSeen > 0 || m_pStorage->m_nRangeLimit >=0)
 		{
 			bool bSetRange(false), bSetIfRange(false);
 
@@ -178,13 +178,29 @@ struct tDlJob
 				bSetRange = true;
 			}
 
+			/*
+			if(m_pStorage->m_nSizeSeen >0 && m_pStorage->m_nRangeLimit>=0)
+			{
+				bool bSaneRange=m_pStorage->m_nRangeLimit >=m_pStorage->m_nSizeSeen;
+				// just to be sure
+				ASSERT(bSaneRange);
+			}
+			if(m_pStorage->m_nRangeLimit < m_pStorage->m_nSizeSeen)
+				bSetRange = bSetIfRange = false;
+*/
+
 			/* use APT's trick - set the starting position one byte lower -
 			 * this way the server has to send at least one byte if the assumed
 			 * position is correct, and we never get a 416 error (one byte
 			 * waste is acceptable).
 			 * */
 			if (bSetRange)
-				head << "Range: bytes=" << (m_pStorage->m_nSizeSeen - 1) << "-\r\n";
+			{
+				head << "Range: bytes=" << std::max(off_t(0), m_pStorage->m_nSizeSeen - 1) << "-";
+				if(m_pStorage->m_nRangeLimit>=0)
+					head << m_pStorage->m_nRangeLimit;
+				head << "\r\n";
+			}
 
 			if (bSetIfRange)
 				head << "If-Range: " << p << "\r\n";
@@ -196,12 +212,12 @@ struct tDlJob
 		if (acfg::exporigin && !xff.empty())
 			head << "X-Forwarded-For: " << xff << "\r\n";
 
-		if (!acfg::requestapx.empty())
-			head.addEscaped(acfg::requestapx.c_str());
-
-		head << "Accept: */*\r\nConnection: Keep-Alive\r\n\r\n";
+		head << acfg::requestapx << "Accept: */*\r\nConnection: Keep-Alive\r\n\r\n";
 
 		ldbg("Request cooked, buffer contents: " << head);
+#ifdef SPAM
+		//head.syswrite(2);
+#endif
 
 	}
 
@@ -478,9 +494,14 @@ inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 inline void dlcon::EnqJob(tDlJob *todo)
 {
 	setLockGuard;
+
+	ASSERT(todo);
+	ASSERT(todo->m_pStorage->m_nRangeLimit <0
+			|| todo->m_pStorage->m_nRangeLimit >= todo->m_pStorage->m_nSizeSeen);
+
 	LOGSTART2("dlcon::EnqJob", todo->m_fileUri.ToURI());
+
 	m_qNewjobs.push_back(tDlJobPtr(todo));
-			
 	if (m_wakepipe[1]>=0)
 		POKE(m_wakepipe[1]);
 }
@@ -493,7 +514,7 @@ void dlcon::AddJob(tFileItemPtr m_pItem,
 
 void dlcon::AddJob(tFileItemPtr m_pItem, tHttpUrl hi)
 {
-	EnqJob(new tDlJob(this, m_pItem, hi.sHost, hi.sPath));
+	EnqJob(new tDlJob(this, m_pItem, hi));
 }
 
 void dlcon::SignalStop()
@@ -570,8 +591,12 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 			dbgline;
 			if (EINTR == errno)
 				continue;
+#ifdef MINIBUILD
+			string fer("select failed");
+#else
 			aclog::errnoFmter fer("FAILURE: select, ");
 			LOG(fer);
+#endif
 			sErrorMsg = string("500 Internal malfunction, ") + fer;
 			return EFLAG_DISCON|EFLAG_JOB_BROKEN|EFLAG_MIRROR_BROKEN;
 		}
@@ -626,8 +651,12 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, int fd, tDljQueue &qActive)
 			{
 				dbgline;
 
+#ifdef MINIBUILD
+				sErrorMsg = "502 EPIC FAIL";
+#else
 				// pickup the error code for later and kill current connection ASAP
 				sErrorMsg = aclog::errnoFmter("502 ");
+#endif
 				return EFLAG_LOST_CON;
 			}
 
