@@ -17,12 +17,12 @@ using namespace MYSTD;
 
 #define MAXTEMPDELAY acfg::maxtempdelay // 27
 
-header const & fileitem::GetHeaderUnlocked()
+header const & fileitem_base::GetHeaderUnlocked()
 {
 	return m_head;
 }
 
-string fileitem::GetHttpMsg()
+string fileitem_base::GetHttpMsg()
 {
 	setLockGuard;
 	if(m_head.frontLine.length()>9)
@@ -30,14 +30,15 @@ string fileitem::GetHttpMsg()
 	return m_head.frontLine;
 }
 
-time_t fileitem::m_nEarliestExpiration(0);
+time_t fileitem_base::m_nEarliestExpiration(0);
 
-fileitem::fileitem(MYSTD::string sPath) :
+fileitem_base::fileitem_base(MYSTD::string sPath) :
 	condition(),
 	m_nIncommingCount(0),
 	m_nSizeSeen(0),
 	m_bCheckFreshness(true),
 	m_bHeadOnly(false),
+	m_nRangeLimit(-1),
 	m_bAllowStoreData(true),
 	m_nSizeChecked(0),
 	m_filefd(-1),
@@ -99,13 +100,38 @@ fileitem::fileitem(MYSTD::string sPath) :
 	//ldbg("item created, m_sPath: "<< m_sPath);
 }
 
-void fileitem::IncDlRefCount()
+
+fileitem_base::fileitem_base() :
+	condition(),
+	m_nIncommingCount(0),
+	m_nSizeSeen(0),
+	m_bCheckFreshness(true),
+	m_bHeadOnly(false),
+	m_nRangeLimit(-1),
+	m_bAllowStoreData(true),
+	m_nSizeChecked(0),
+	m_filefd(-1),
+	m_nDlRefsCount(0),
+	m_status(FIST_FRESH),
+	m_nTimeExpireAt(0)
+{
+}
+
+fileitem_base::~fileitem_base()
+{
+	setLockGuard;
+//	m_head.clear();
+	checkforceclose(m_filefd);
+}
+
+
+void fileitem_base::IncDlRefCount()
 {
 	setLockGuard;
 	m_nDlRefsCount++;
 }
 
-void fileitem::DecDlRefCount(const string &sReason)
+void fileitem_base::DecDlRefCount(const string &sReason)
 {
 	setLockGuard;
 	
@@ -124,13 +150,13 @@ void fileitem::DecDlRefCount(const string &sReason)
 		m_head.frontLine=string("HTTP/1.1 ")+sReason;
 		m_head.type=header::ANSWER;
 
-		if (acfg::debug>1)
+		if (acfg::debug&LOG_MORE)
 			aclog::misc(string("Download of ")+m_sPathRel+" aborted");
 	}
 	checkforceclose(m_filefd);
 }
 
-uint64_t fileitem::GetTransferCount()
+uint64_t fileitem_base::GetTransferCount()
 {
 	setLockGuard;
 	uint64_t ret=m_nIncommingCount;
@@ -138,17 +164,16 @@ uint64_t fileitem::GetTransferCount()
 	return ret;
 }
 
-int fileitem::GetFileFd() {
+int fileitem_base::GetFileFd() {
 	LOGSTART("fileitem::GetFileFd");
 	setLockGuard;
-	if(m_status<FIST_DLGOTHEAD)
-		return -1;
+
 	ldbg("Opening " << m_sPathAbs);
 	int fd=open(m_sPathAbs.c_str(), O_RDONLY);
 	
 #ifdef HAVE_FADVISE
 	// optional, experimental
-	if(m_status==FIST_COMPLETE)
+	if(fd>=0)
 		posix_fadvise(fd, 0, m_nSizeChecked, POSIX_FADV_SEQUENTIAL);
 #endif
 
@@ -162,7 +187,7 @@ off_t GetFileSize(cmstring & path, off_t defret)
 }
 
 
-void fileitem::ResetCacheState()
+void fileitem_base::ResetCacheState()
 {
 	setLockGuard;
 	m_nSizeSeen = 0;
@@ -172,7 +197,7 @@ void fileitem::ResetCacheState()
 	m_head.clear();
 }
 
-FiStatus fileitem::Setup(bool bCheckFreshness)
+FiStatus fileitem_base::Setup(bool bCheckFreshness)
 {
 	LOGSTART2("fileitem::Setup", bCheckFreshness);
 
@@ -211,21 +236,21 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 				// file larger than it could ever be?
 				if(nContLen < m_nSizeSeen)
 					goto error_clean;
-				
-				LOG("Content-Length has a sane range");
 
-				// is it complete? and 0 value also looks weird, try to verify
+
+				LOG("Content-Length has a sane range");
+				
+				m_nSizeChecked=m_nSizeSeen;
+
+				// is it complete? and 0 value also looks weird, try to verify later
 				if(m_nSizeSeen == nContLen && nContLen>0)
-				{
 					m_status=FIST_COMPLETE;
-					m_nSizeChecked=m_nSizeSeen;
-				}
 			}
 			else
 			{
 				// no content length known, assume it's ok
 				m_nSizeChecked=m_nSizeSeen;
-			}				
+			}
 		}
 	}
 	else // -> no .head file
@@ -246,17 +271,30 @@ FiStatus fileitem::Setup(bool bCheckFreshness)
 			return m_status; // unuseable, to be redownloaded
 }
 
+bool fileitem_base::CheckUsableRange_unlocked(off_t nRangeLastByte)
+{
+	if(m_status == FIST_COMPLETE)
+		return true;
+	if(m_status < FIST_INITED || m_status > FIST_COMPLETE)
+		return false;
+	if(m_status >= FIST_DLGOTHEAD)
+		return nRangeLastByte > m_nSizeChecked;
 
-bool fileitem::SetupClean()
+	// special exceptions for static files
+	return (m_status == FIST_INITED && !m_bCheckFreshness
+			&& m_nSizeSeen>0 && nRangeLastByte >=0 && nRangeLastByte <m_nSizeSeen
+			&& atoofft(m_head.h[header::CONTENT_LENGTH], -255) > nRangeLastByte);
+}
+
+bool fileitem_base::SetupClean()
 {
 	setLockGuard;
 
 	if(m_status>FIST_FRESH)
 		return false;
 	m_status=FIST_INITED;
-
-	unlink(m_sPathAbs.c_str());
-	unlink((m_sPathAbs+".head").c_str());
+	::truncate(m_sPathAbs.c_str(), 0);
+	::truncate((m_sPathAbs+".head").c_str(), 0);
 	m_head.clear();
 	m_nSizeSeen=m_nSizeChecked=0;
 
@@ -269,7 +307,9 @@ inline void _LogWithErrno(const char *msg, const string & sFile)
 	aclog::err( (sFile+" storage error ["+msg+"]: "+f).c_str());
 }
 
-bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextData)
+#ifndef MINIBUILD
+
+bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
 
@@ -314,7 +354,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	}
 	case 206:
 	{
-		if(m_nSizeSeen<=0)
+		if(m_nSizeSeen<=0 && m_nRangeLimit<0)
 		{
 			// wtf? Cannot have requested partial content
 			BOUNCE("500 Unexpected Partial Response");
@@ -336,9 +376,13 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		ldbg("resuming? n: "<< n << " und myfrom: " <<myfrom << 
 				" und myto: " << myto << " und mylen: " << mylen);
 		if(n!=3  // check for nonsense
-				|| myfrom != m_nSizeSeen-1
-				|| myfrom<0 || mylen<0)
-			BOUNCE("500 Server reports illegal range");
+				|| (m_nSizeSeen>0 && myfrom != m_nSizeSeen-1)
+				|| (m_nRangeLimit>=0 && myto != m_nRangeLimit)
+				|| myfrom<0 || mylen<0
+		)
+		{
+			BOUNCE("500 Server reports unexpected range");
+		}
 	
 		m_nSizeChecked=myfrom;
 		
@@ -398,7 +442,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		}
 	}
 
-	if(acfg::debug>1)
+	if(acfg::debug&LOG_MORE)
 		aclog::misc(string("Download of ")+m_sPathRel+" started");
 
 	if(m_bAllowStoreData)
@@ -507,7 +551,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 	return false;
 }
 
-bool fileitem::StoreFileData(const char *data, unsigned int size)
+bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 {
 
 	setLockGuard;
@@ -532,7 +576,7 @@ bool fileitem::StoreFileData(const char *data, unsigned int size)
 
 		m_status=FIST_COMPLETE;
 
-		if(acfg::debug>1)
+		if(acfg::debug&LOG_MORE)
 			aclog::misc(tSS()<<"Download of "<<m_sPathRel<<" finished");
 		
 		// we are done! Fix header from chunked transfers?
@@ -571,13 +615,6 @@ bool fileitem::StoreFileData(const char *data, unsigned int size)
 	return true;
 }
 
-fileitem::~fileitem()
-{
-	setLockGuard;
-	m_head.clear();
-	checkforceclose(m_filefd);
-}
-
 
 struct tFileRefEntry
 {
@@ -655,7 +692,7 @@ tFileItemPtr fileitem::GetRegisteredFileItem(MYSTD::string sPathKey)
 }
 
 bool fileitem::RegisterFileItem(mstring sPathRel,
-		tFileItemPtr spCustomFileItem)
+		SHARED_PTR<fileitem> spCustomFileItem)
 		{
 			if (sPathRel.empty() || !spCustomFileItem)
 				return false;
@@ -753,7 +790,25 @@ void DumpItems()
 }
 #endif
 
-FiStatus fileitem::WaitForFinish(int *httpCode)
+
+
+ssize_t fileitem_with_storage::SendData(int out_fd, int in_fd, off_t &nSendPos, size_t count)
+{
+#ifndef HAVE_LINUX_SENDFILE
+	return sendfile_generic(out_fd, in_fd, &nSendPos, count);
+#else
+	ssize_t r=sendfile(out_fd, in_fd, &nSendPos, count);
+
+	if(r<0 && (errno==ENOSYS || errno==EINVAL))
+		return sendfile_generic(out_fd, in_fd, &nSendPos, count);
+	else
+		return r;
+#endif
+}
+
+#endif // MINIBUILD
+
+FiStatus fileitem_base::WaitForFinish(int *httpCode)
 {
 	setLockGuard;
 	while(m_status<FIST_COMPLETE)
