@@ -339,3 +339,21 @@ MYSTD::string header::GenInfoHeaders()
 	    ret+="\r\nServer: Debian Apt-Cacher NG/"ACVERSION"\r\n";
 	    return ret;
 }
+
+static const char* fmts[] =
+{
+		"%a, %d %b %Y %H:%M:%S GMT",
+		"%A, %d-%b-%y %H:%M:%S GMT",
+		"%a %b %d %H:%M:%S %Y"
+};
+
+bool header::ParseDate(const char *s, struct tm *tm)
+{
+	if(!s || !tm)
+		return false;
+	for(const char **p=fmts; p<fmts+_countof(fmts); ++p)
+		if(::strptime(s, *p, tm))
+			return true;
+
+	return false;
+}
