@@ -18,10 +18,6 @@ using namespace MYSTD;
 #include <algorithm>
 #include "maintenance.h"
 
-#ifdef HAVE_LINUX_SENDFILE
-#include <sys/sendfile.h>
-#endif
-
 #include <errno.h>
 
 // XXX no longer needed after debug logging overhaul
@@ -37,7 +33,7 @@ inline bool IsValidFD(int fd) { return fd>=0 || SPECIAL_FD == fd; }
  * harddisk. Instead, it uses the download buffer and lets job object send the data straight from
  * it to the client.
  */
-class tPassThroughFitem : public fileitem
+class tPassThroughFitem : public fileitem_base
 {
 protected:
 
@@ -45,7 +41,7 @@ protected:
 	size_t m_nConsumable, m_nConsumed;
 
 public:
-	tPassThroughFitem(MYSTD::string s) : fileitem(s),
+	tPassThroughFitem(MYSTD::string s) : fileitem_base(s),
 	m_pData(NULL), m_nConsumable(0), m_nConsumed(0)
 	{
 		m_bAllowStoreData=false;
@@ -150,14 +146,15 @@ public:
 };
 
 // fileitem replacement with custom storage for local data
-class tGeneratedFitemBase : public fileitem
+class tGeneratedFitemBase : public fileitem_base
 {
 public:
 	virtual void Unreg() {}; // NOOP, since this item is not shared
 	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
 	tSS m_data;
 
-	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg) : fileitem(sFitemId), m_data(256)
+	tGeneratedFitemBase(const string &sFitemId, const char *szFrontLineMsg)
+	: fileitem_base(sFitemId), m_data(256)
 	{
 		m_status=FIST_COMPLETE;
 		m_head.type = header::ANSWER;
@@ -179,6 +176,9 @@ public:
 		m_nSizeChecked = m_data.size();
 		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 	}
+	// never used to store data
+	bool DownloadStartedStoreHeader(const header &, const char *) {return false;};
+	bool StoreFileData(const char *, unsigned int) {return false;};
 };
 
 ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
@@ -226,20 +226,6 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 	return totalcnt;
 }
 
-ssize_t fileitem::SendData(int out_fd, int in_fd, off_t &nSendPos, size_t count)
-{
-#ifndef HAVE_LINUX_SENDFILE
-	return sendfile_generic(out_fd, in_fd, &nSendPos, count);
-#else
-	ssize_t r=sendfile(out_fd, in_fd, &nSendPos, count);
-
-	if(r<0 && (errno==ENOSYS || errno==EINVAL))
-		return sendfile_generic(out_fd, in_fd, &nSendPos, count);
-	else
-		return r;
-#endif
-}
-
 
 job::job(header *h, con *pParent) :
 	m_filefd(-1),
@@ -247,14 +233,16 @@ job::job(header *h, con *pParent) :
 	m_bChunkMode(false),
 	//m_bCloseCon(false),
 	m_bIsHttp11(false),
+	m_bNoDownloadStarted(false),
 	m_state(STATE_SEND_MAIN_HEAD),
 	m_backstate(STATE_TODISCON),
 	m_pReqHead(h),
 	m_nSendPos(0),
-	m_nRangeLast(MAX_VAL(off_t)-1), // range size should never overflow
+	m_nCurrentRangeLast(MAX_VAL(off_t)-1), // range size should never overflow
 	m_nAllDataCount(0),
 	m_nChunkRemainingBytes(0),
-	m_type(FILE_INVALID)
+	m_type(FILE_INVALID),
+	m_nRangeFrom(-1), m_nRangeTo(-1)
 {
 	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << uintptr_t(this));
 }
@@ -365,7 +353,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 					seal();
 				}
 			};
-			m_pItem.reset(static_cast<fileitem*>(new dirredirect(visPath)));
+			m_pItem.reset(static_cast<fileitem_base*>(new dirredirect(visPath)));
 			return;
 		}
 
@@ -379,7 +367,7 @@ inline void job::HandleLocalDownload(const string &visPath,
 			}
 		};
 		listing *p=new listing(visPath);
-		m_pItem.reset(static_cast<fileitem*>(p)); // assign to smart pointer ASAP, operations might throw
+		m_pItem.reset(static_cast<fileitem_base*>(p)); // assign to smart pointer ASAP, operations might throw
 		tSS & page = p->m_data;
 
 		page << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">"
@@ -442,10 +430,10 @@ inline void job::HandleLocalDownload(const string &visPath,
 	 * This variant of file item handler sends a local file. The
 	 * header data is generated as needed.
 	 */
-	class tLocalGetFitem : public fileitem
+	class tLocalGetFitem : public fileitem_with_storage
 	{
 	public:
-		tLocalGetFitem(string s, string sLocalPath, struct stat &stdata) : fileitem(s)
+		tLocalGetFitem(string s, string sLocalPath, struct stat &stdata) : fileitem_with_storage(s)
 		{
 			m_bAllowStoreData=false;
 			m_sPathAbs=sLocalPath;
@@ -461,7 +449,40 @@ inline void job::HandleLocalDownload(const string &visPath,
 		};
 		virtual void Unreg() {}; // NOOP, since this item is not shared
 	};
-	m_pItem.reset(static_cast<fileitem*>(new tLocalGetFitem(visPath, absPath, stbuf)));
+	m_pItem.reset(static_cast<fileitem_base*>(new tLocalGetFitem(visPath, absPath, stbuf)));
+}
+
+inline bool job::ParseRange()
+{
+
+	/*
+	 * Range: bytes=453291-
+	 * ...
+	 * Content-Length: 7271829
+	 * Content-Range: bytes 453291-7725119/7725120
+	 */
+
+	const char *pRange = m_pReqHead->h[header::RANGE];
+	// working around a bug in old curl versions
+	if (!pRange)
+		pRange = m_pReqHead->h[header::CONTENT_RANGE];
+	if (pRange)
+	{
+		int nRangeItems = sscanf(pRange, "bytes=" OFF_T_FMT
+		"-" OFF_T_FMT, &m_nRangeFrom, &m_nRangeTo);
+		// working around bad (old curl style) requests
+		if (nRangeItems <= 0)
+		{
+			nRangeItems = sscanf(pRange, "bytes "
+			OFF_T_FMT "-" OFF_T_FMT, &m_nRangeFrom, &m_nRangeTo);
+		}
+
+		if (nRangeItems < 1) // weird...
+			m_nRangeFrom = m_nRangeTo = -2;
+		else
+			return true;
+	}
+	return false;
 }
 
 void job::PrepareDownload() {
@@ -521,12 +542,21 @@ void job::PrepareDownload() {
 		}
 		LOG("refined path: " << tUrl.sPath);
 
-		if(!tUrl.sPort.empty() && tUrl.sPort!="80")
+		char *pEnd(0);
+		UINT nPort = 80;
+		if(!tUrl.sPort.empty())
 		{
-			//USRDBG(3, "illegal port " << tUrl.sPort << "?" << tUrl.sHost << tUrl.sPath);
-			if(acfg::port == tUrl.sPort)
-				goto report_doubleproxy;
+			nPort = (UINT) strtoul(tUrl.sPort.c_str(), &pEnd, 10);
+			if('\0' != *pEnd || pEnd == tUrl.sPort.c_str() || nPort > TCP_PORT_MAX || !nPort)
+				goto report_invport;
+		}
 
+		// if no filter set, it must be 80; with filter, it must be therein
+		if( (acfg::pUserPorts && ! acfg::pUserPorts->test(nPort)) // alarm
+				|| (!acfg::pUserPorts && nPort != 80))
+		{
+			if(acfg::port == tUrl.sPort) // admin failed to RTFM?
+				goto report_doubleproxy;
 			goto report_invport;
 		}
 
@@ -609,9 +639,25 @@ void job::PrepareDownload() {
 	if (bPtMode && fistate != FIST_COMPLETE)
 		fistate = _SwitchToPtItem(m_sFileLoc);
 
+	ParseRange();
+
 	if(fistate==FIST_COMPLETE)
 		return; // perfect, done here
 
+	// early optimization for requests which want a defined file part which we already have
+	// no matter whether the file is complete or not
+	// handles different cases: GET with range, HEAD with range, HEAD with no range
+	if((m_nRangeFrom>=0 && m_nRangeTo>=0) || (m_pReqHead->type==header::HEAD && 0!=(m_nRangeTo=-1)))
+	{
+		lockguard g(*m_pItem);
+		if(m_pItem->CheckUsableRange_unlocked(m_nRangeTo))
+		{
+			LOG("Got a partial request for incomplete download; range is available");
+			m_bNoDownloadStarted=true;
+			return;
+		}
+	}
+
     if(acfg::offlinemode) { // make sure there will be no problems later in SendData or prepare a user message
     	// error or needs download but freshness check was disabled, so it's really not complete.
     	goto report_offlineconf;
@@ -682,7 +728,7 @@ report_invpath:
     return ;
 
 report_invport:
-	SetErrorResponse("403 Prohibited port (configure Remap-...)");
+	SetErrorResponse("403 Prohibited port or config error (double proxy)");
     return ;
 
 report_doubleproxy:
@@ -703,7 +749,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 			return R_DISCON; // just stop and close connection
 	}
 	
-	off_t nGooddataSize(0);
+	off_t nGoodDataSize(0);
 	FiStatus fistate(FIST_LOCALERROR);
 	header respHead; // template of the response header, e.g. as found in cache
 
@@ -714,9 +760,9 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 	{
 		lockguard g(*m_pItem);
 		
-		while (true)
+		for(;;)
 		{
-			fistate=m_pItem->GetStatusUnlocked(nGooddataSize);
+			fistate=m_pItem->GetStatusUnlocked(nGoodDataSize);
 			
 			dbgline;
 			if (fistate>=FIST_LOCALERROR)
@@ -734,9 +780,19 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 					SetErrorResponse("500 Unknown error");
 				return R_AGAIN;
 			}
-			
+			/*
+			 * Detect the same special case as above. There is no download agent to change
+			 * the state so no need to wait.
+			*/
+			if(m_bNoDownloadStarted)
+			{
+				fistate = FIST_DLRECEIVING;
+				break;
+			}
+
+			// or wait for the dl source to get data at the position we need to start from
 			dbgline;
-			if(fistate==FIST_COMPLETE || (m_nSendPos < nGooddataSize && fistate>=FIST_DLGOTHEAD))
+			if(fistate==FIST_COMPLETE || (m_nSendPos < nGoodDataSize && fistate>=FIST_DLGOTHEAD))
 				break;
 			
 			dbgline;
@@ -757,11 +813,15 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 		return R_DISCON;
 	}
 
-	USRDBG(10, "set/restoring state, fileitem " << size_t(m_pItem.get())
-			<< " with fistate " << fistate << " and procstate " << m_state );
-
-	while (true) // left by returning
+	for(;;) // left by returning
 	{
+
+		/*
+		USRDBG(10,  m_sFileLoc << " : to state " << m_state
+				<< " and bs: " << m_backstate <<
+				" with fistate " << fistate << " sendpos: " << m_nSendPos
+				<< " goodsize: " << nGoodDataSize);
+*/
 		MYTRY // for bad_alloc in members
 		{
 			switch(m_state)
@@ -773,7 +833,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						return R_AGAIN;
 					m_state=STATE_SEND_BUFFER;
 					m_backstate=STATE_HEADER_SENT; // could be changed while creating header
-					const char *szErr = BuildAndEnqueHeader(fistate, nGooddataSize, respHead);
+					const char *szErr = BuildAndEnqueHeader(fistate, nGoodDataSize, respHead);
 					if(szErr) THROW_ERROR(szErr);
 					USRDBG(REPLEVEL, "Prepared response header for user: \n" << m_sendbuf );
 					return R_AGAIN;
@@ -797,11 +857,11 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				}
 				case(STATE_SEND_PLAIN_DATA):
 				{
-					ldbg("STATE_SEND_PLAIN_DATA, max. " << nGooddataSize);
+					ldbg("STATE_SEND_PLAIN_DATA, max. " << nGoodDataSize);
 
 					// eof?
 #define GOTOENDE { m_state=STATE_FINISHJOB ; continue; }
-					if(m_nSendPos>=nGooddataSize)
+					if(m_nSendPos>=nGoodDataSize)
 					{
 						if(fistate>=FIST_COMPLETE)
 							GOTOENDE;
@@ -809,7 +869,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						return R_AGAIN;
 					}
 
-					size_t nMax2SendNow=min(nGooddataSize-m_nSendPos, m_nRangeLast+1-m_nSendPos);
+					size_t nMax2SendNow=min(nGoodDataSize-m_nSendPos, m_nCurrentRangeLast+1-m_nSendPos);
 					ldbg("~sendfile: on "<< m_nSendPos << " up to : " << nMax2SendNow);
 					int n = m_pItem->SendData(confd, m_filefd, m_nSendPos, nMax2SendNow);
 					ldbg("~sendfile: " << n << " new m_nSendPos: " << m_nSendPos);
@@ -818,7 +878,8 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 						m_nAllDataCount+=n;
 
 					// shortcuts
-					if(m_nSendPos>m_nRangeLast || (fistate==FIST_COMPLETE && m_nSendPos==nGooddataSize))
+					if(m_nSendPos>m_nCurrentRangeLast ||
+							(fistate==FIST_COMPLETE && m_nSendPos==nGoodDataSize))
 						GOTOENDE;
 					
 					if(n<0)
@@ -829,7 +890,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				}
 				case(STATE_SEND_CHUNK_HEADER):
 				{
-					m_nChunkRemainingBytes=nGooddataSize-m_nSendPos;
+					m_nChunkRemainingBytes=nGoodDataSize-m_nSendPos;
 					ldbg("STATE_SEND_CHUNK_HEADER for " << m_nChunkRemainingBytes);
 					m_sendbuf << tSS::hex << m_nChunkRemainingBytes << tSS::dec
 							<< (m_nChunkRemainingBytes ? "\r\n" : "\r\n\r\n");
@@ -984,81 +1045,63 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			// we deal with them equally but need to know which to use
 			const char *pIfmo = m_pReqHead->h[header::RANGE] ?
 					m_pReqHead->h[header::IFRANGE] : m_pReqHead->h[header::IF_MODIFIED_SINCE];
+			const char *pLastMo = respHead.h[header::LAST_MODIFIED];
 
 			// consider contents "fresh" for non-volatile data, or when "our" special client is there, or the client simply doesn't care
 			bool bFreshnessForced = (m_type != FILE_INDEX
 				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
 			// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
-			// TODO: add format tolerant time comparer
-			bool bIfModSeenAndChecked=(pIfmo && respHead.h[header::LAST_MODIFIED] &&
-					0==strcmp(pIfmo, respHead.h[header::LAST_MODIFIED]));
+			struct tm tm1, tm2;
+			bool bIfModSeenAndChecked=false;
+			if(pIfmo && header::ParseDate(pIfmo, &tm1) && header::ParseDate(pLastMo, &tm2))
+			{
+				time_t a(mktime(&tm1)), b(mktime(&tm2));
+				LOG("if-mo-since: " << a << " vs. last-mo: " << b);
+				bIfModSeenAndChecked = (a==b);
+			}
 
 			// is it fresh? or is this relevant? or is range mode forced?
 			if(  bFreshnessForced || bIfModSeenAndChecked)
 			{
 				off_t nContLen=atoofft(respHead.h[header::CONTENT_LENGTH]);
 
-				/*
-				 * Range: bytes=453291-
-				 * ...
-				 * Content-Length: 7271829
-				 * Content-Range: bytes 453291-7725119/7725120
-				 */
-
-				const char *pRange=m_pReqHead->h[header::RANGE];
-
-				// working around a bug in old curl versions
-				if(!pRange)
-					pRange=m_pReqHead->h[header::CONTENT_RANGE];
-
-				if(pRange)
+				if(m_nRangeFrom >=0)
 				{
-					off_t nRangeFrom(0), nRangeTo(0);
-					int nRangeItems=sscanf(pRange, "bytes=" OFF_T_FMT
-                            "-" OFF_T_FMT, &nRangeFrom, &nRangeTo);
-                    // working around bad (old curl style) requests
-                    if(nRangeItems <= 0)
-                    {
-                        nRangeItems=sscanf(pRange, "bytes "
-                                OFF_T_FMT "-" OFF_T_FMT,
-                                &nRangeFrom, &nRangeTo);
-                    }
-					if(nRangeItems == 1) // open-end? set the limit
-						nRangeTo=nContLen-1;
+					if(m_nRangeTo<0 || m_nRangeTo>=nContLen) // open-end? set the end to file length. Also when request range would be too large
+						m_nRangeTo=nContLen-1;
 
 					/*
 					 * make sure that our client doesn't just hang here while the download thread is
 					 * fetching from 0 to start position for many minutes. If the resumed position
 					 * is beyond of what we already have, fall back to 200 (complete download).
 					 */
-					if(nRangeItems>0
-						&&
-						(  fistate==FIST_COMPLETE
-							// or can start sending within this range
+					if(  fistate==FIST_COMPLETE
+							// or can start sending within this range (positive range-from)
 							|| (
 									fistate >= FIST_DLGOTHEAD
 									&& fistate <= FIST_COMPLETE
-									&& nGooddataSize>=nRangeFrom
+									&& nGooddataSize>=m_nRangeFrom
 								)
 							// don't care, found special hint from acngfs (kludge...)
 							|| m_pReqHead->h[header::ACNGFSMARK]
-							)
+
 						)
 					{
-						USRDBG(20, "request range problem: ")
+						USRDBG(20, "request range problem: ");
+
 						// detect errors, out-of-range case
-						if(nRangeTo>=nContLen || nRangeFrom>=nContLen || nRangeTo<nRangeFrom)
+						if(m_nRangeFrom>=nContLen || m_nRangeTo<m_nRangeFrom)
 							return "416 Requested Range Not Satisfiable";
 
-						m_nSendPos = nRangeFrom;
-						m_nRangeLast = nRangeTo;
+						m_nSendPos = m_nRangeFrom;
+						m_nCurrentRangeLast = m_nRangeTo;
 						// replace with partial-response header
 						sb.clear();
 						sb << "HTTP/1.1 206 Partial Response\r\nContent-Length: "
-						 << (m_nRangeLast-m_nSendPos+1) <<
+						 << (m_nCurrentRangeLast-m_nSendPos+1) <<
 								"\r\nContent-Range: bytes "<< m_nSendPos
-								<< "-" << m_nRangeLast << "/" << nContLen << "\r\n";
+								<< "-" << m_nCurrentRangeLast << "/" << nContLen << "\r\n";
 						bGotLen=true;
 					}
 				}
@@ -1145,7 +1188,7 @@ void job::SetErrorResponse(const char * errorLine)
 		}
 	};
 
-	tFileItemPtr pNew(static_cast<fileitem*>(new erroritem("noid", errorLine)));
+	tFileItemPtr pNew(static_cast<fileitem_base*>(new erroritem("noid", errorLine)));
 
 	fileitem::UnregOneASAP(m_pItem);
 
