@@ -363,7 +363,7 @@ tStrDeq ExpandFilePattern(const string &pattern, bool bSorted)
 
 #ifndef MINIBUILD
 
-void MakeAbsolutePath(MYSTD::string &dirToFix, const MYSTD::string &reldir)
+bool IsAbsolute(cmstring &dirToFix)
 {
 	bool bAbs=false;
 #ifdef WIN32
@@ -373,10 +373,16 @@ void MakeAbsolutePath(MYSTD::string &dirToFix, const MYSTD::string &reldir)
 #else
 	bAbs = (!dirToFix.empty() && CPATHSEPUNX==dirToFix[0]);
 #endif
-	if(!bAbs)
-		dirToFix=reldir+CPATHSEP+dirToFix;
+	return bAbs;
 }
 
+/*
+void MakeAbsolutePath(MYSTD::string &dirToFix, const MYSTD::string &reldir)
+{
+	if(!IsAbsolute(dirToFix))
+		dirToFix=reldir+CPATHSEP+dirToFix;
+}
+*/
 
 void HttpEscape(string &s)
 {
